
local Object = require("source.Object")
local Actor = Object:extend()

local screenWidth, screenHeight = love.graphics.getDimensions()

function Actor:new(x, y)
  self.super.new(Actor)
  self:setDimensions(32, 64)
  self:setPosition(x, y)
  self:setColor({250, 50, 50})
  self.velocity = {x=0, y=0, transitionX=20, transitionY=20, limitX=800, limitY=800}
  self.speed    = {x=250, y=300}
  self.onGround = false
  self.type     = "Actor"
  self.collidable = true
  self:addToCollisionWorld()
end


function Actor:updateKeyMovement(dt)
  local keyDown = love.keyboard.isDown
  local up, down, left, right, jump = keyDown("up"), keyDown("down"), keyDown("left"), keyDown("right"), keyDown("space")
  
  if left then self:accelerate(-self.speed.x, 0) end
  if right then self:accelerate(self.speed.x, 0) end
  if not left and not right then self.velocity.x = self.velocity.x * (1 - dt * 7.5) end
  self:move(self.velocity.x*dt, self.velocity.y*dt)
  
  -- Jumping
  if jump and self.onGround then
    self.velocity.y = -375
  end
end


function Actor:applyGravity(dt)
  if not self.onGround then
    if math.abs(self.velocity.y) < self.velocity.limitY then self.velocity.y = self.velocity.y+750*dt end
    if self.y > 1000 then self:destroy() end
  end
end


function Actor:filter(other)
  if other.type == "Wall" then return "slide" end
  if other.type == "Portal" then return "cross" end
end


function Actor:resolveCollision(other)
  if other.type == "Wall" then
    -- Ground collision
    if self.y+self.height <= other.y then
      self.onGround = true
      self.velocity.y = 0
    end
  elseif other.type == "Portal" then
    other:teleport(self)
  end
end


function Actor:update(dt)
  self:updateKeyMovement(dt)
  self:applyGravity(dt)
end


function Actor:draw()
  self:drawRectangle("fill")
end

return Actor

