
local Object  = require("source.classic")
local Game    = Object:extend()

local Container = require("source.Container")
local Camera    = require("source.camera")
local Bump      = require("source.bump")
local Actor     = require("source.Actor")
local Wall      = require("source.Wall")
local Portal    = require("source.Portal")

local screenWidth, screenHeight = love.graphics.getDimensions()
local actor
local objectWorld
local collisionWorld
local camera

function Game:init()
  self.objectWorld = Container()
  self.collisionWorld = Bump.newWorld()
  actor = self.objectWorld:add(Actor(300, 100))
  camera = Camera(actor.x, actor.y)
  --camera:zoomTo(.75)
  
  -- Temp Walls for testing --
  self.objectWorld:add(Wall(300, 380, 200, 8))
  self.objectWorld:add(Wall(screenWidth-500, 380, 200, 8))
  
  -- Temp portals --
  local p1, p2
  p1 = self.objectWorld:add(Portal(650, 280))
  p2 = self.objectWorld:add(Portal(650, 80))
  p1:connect(p2)
  p2:connect(p1)
  
end


function Game:getObjectWorld()
  return self.objectWorld
end


function Game:getCollisionWorld()
  return self.collisionWorld
end


function Game:update(dt)
  camera:lockPosition(actor.x, actor.y, Camera.smooth.damped(5))
  self.objectWorld:update(dt)
end


function Game:draw()
  camera:attach()
  self.objectWorld:draw()
  camera:detach()
  love.graphics.print("Move with Arrow Keys", 10, 20)
  love.graphics.print("Jump with SPACE", 10, 40)
  love.graphics.print("R to Reset", 10, 60)
  love.graphics.print("ESC to quit", 10, 80)
end


function Game:keypressed(key)
  if key == "r" then love.load() end
  if key == "escape" then love.event.quit() end
end

return Game

