
local classic = require("source.classic")
local Object = classic:extend()


local addToCollisionWorld = function(self, x, y, width, height)
  self.collisionWorld = require("source.Game"):getCollisionWorld()
  self.collisionWorld:add(self, x or self.x, y or self.y, width or self.width, height or self.height)
end


local updateCollisionWorld = function(self)
  if not self.collidable then return end
  if self.collisionWorld:hasItem(self) then self.collisionWorld:update(self, self.x, self.y, self.width, self.height)
  else print("Unable to update collision world for Obj ID'"..tostring(self.type)..", "..tostring(self.id).."': object not found!") end
end


function Object:new(x, y)
  self.type = "Object"
  self:setDimensions()
  self:setPosition(x, y)
  self:setColor()
  self.priority   = 0
end


function Object:addToCollisionWorld(x, y, width, height)
  if not self.collidable then error("Collidable variable false or not set!")
  else addToCollisionWorld(self, x, y, width, height) end
end


function Object:setPosition(x, y)
  self.x = x or self.x or 0
  self.y = y or self.y or 0
  updateCollisionWorld(self)
end


function Object:setDimensions(w, h)
  self.width = w or self.width or 16
  self.height = h or self.height or 16
  updateCollisionWorld(self)
end


function Object:getPosition()
  return self.x, self.y
end


function Object:getDimensions()
  return self.width, self.height
end


function Object:setColor(color)
  self.color = color or self.color or {255, 255, 255, 255}
end


function Object:setPriority(v)
  self.priority = v or self.priority
  local world = require("source.Game"):getObjectWorld()
  if world then world:order() end
end


function Object:move(dx, dy)
  local newx, newy = self.x + dx, self.y + dy
  if self.x == newx and self.y == newy then return end
  if self.velocity.y ~= 0 then self.onGround = false end
  if self.collidable then newx, newy = self:processCollisions(newx, newy) end
  self.x = newx
  self.y = newy
end


function Object:destroy()
  if self.collidable then self.collisionWorld:remove(self) end
  self.remove = true
end


function Object:filter()
  return "slide"
end


function Object:processCollisions(x, y)
  local newx, newy = x, y
  local wx, wy, cols, len = self.collisionWorld:move(self, newx, newy, self.filter)
  if len > 0 then 
    for i=1, #cols do
      local other = cols[i].other
      if self.resolveCollision and other.collidable then
        self:resolveCollision(other)
      end
    end
    newx = wx
    newy = wy
  else
    self.onGround = false
  end

  return newx, newy
end


function Object:accelerate(vx, vy)
  local dt = love.timer.getDelta()
  if vx ~= 0 then self.velocity.x = self.velocity.x * (1 - dt * self.velocity.transitionX) + vx * (dt * self.velocity.transitionX) end
  if vy ~= 0 then self.velocity.y = self.velocity.y * (1 - dt * self.velocity.transitionY) + vy * (dt * self.velocity.transitionY) end
end


function Object:addAnimation(name, animation)
  if not self.animations then 
    self.animations = {}
  end
  self.currentAnimation = animation:clone()
  self.animations[name] = self.currentAnimation
  return self.currentAnimation
end


function Object:setAnimation(name)
  if self.currentAnimation == name then return end
  self.currentAnimation = self.animations[name]
end


function Object:updateAnimation(dt)
  if self.currentAnimation then self.currentAnimation:update(dt) end
end


function Object:drawAnimation(spritesheet, ...)
  love.graphics.setColor(self.color)
  if self.currentAnimation and self.visible then
    self.currentAnimation:draw(spritesheet, ...)
  end
  love.graphics.setColor(255, 255, 255, 255)
end


function Object:drawRectangle(mode)
  love.graphics.setColor(self.color)
  love.graphics.rectangle(mode or "fill", self.x, self.y, self.width, self.height)
  love.graphics.setColor(255, 255, 255, 255)
end


function Object:drawCircle(mode)
  love.graphics.setColor(self.color)
  love.graphics.rectangle(mode or "fill", self.x, self.y, self.width, self.height)
  love.graphics.circle(mode or "fill", self.x+self.width/2, self.y+self.width/2, self.width)
  love.graphics.setColor(255, 255, 255, 255)
end


function Object:drawSprite(...)
  love.graphics.setColor(self.color)
  love.graphics.draw(...)
  love.graphics.setColor(255, 255, 255, 255)
end


function Object:drawDebugText()
  love.graphics.printf(self.type, self.x-self.width/2, self.y-font_height-2, self.width*2, "center")
  love.graphics.printf("ID: "..self.id, self.x-self.width/2, self.y+self.height+2, self.width*2, "center")
end


return Object

