

local Object = require("source.Object")
local Portal = Object:extend()

function Portal:new(x, y, w, h)
  self.super.new(Portal)
  self:setPosition(x, y)
  self:setDimensions(w, h)
  self.type = "Portal"
  self.collidable = true
  self.destination = nil -- other portal
  self.used = false
  self.cooldown = 1
  self.timer = 0
  self:addToCollisionWorld(self.x+self.width/3, self.y+self.height/3, self.width/3, self.height/3)
end


function Portal:connect(portal)
  self.destination = portal
end


function Portal:teleport(object)
  local dx, dy = self.destination:getPosition()
  local dw, dh = self.destination:getDimensions()
  
  if self.used then return end
  self:use()
  if self.destination then self.destination:use() end
  object:setPosition(dx+dw/2 - object.width/2, dy+dh/2 - object.height/2)
end


function Portal:use()
  self.used = true
  self.timer = self.cooldown
end


function Portal:update(dt)
  if self.used then
    self.timer = self.timer - dt
    if self.timer <= 0 then
      self.used = false
    end
  end
end


function Portal:draw()
  self:drawCircle("line")
  love.graphics.setColor(255, 255, 255)
  if self.used then love.graphics.setColor(255, 0, 0) end
  love.graphics.rectangle("fill", self.x+self.width/3, self.y+self.height/3, self.width/3, self.height/3)
  love.graphics.setColor(255, 255, 255)
end

return Portal


